﻿using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;
using System.IO;
using System.Threading;

namespace Rs232Api
{

    /// <summary>
    /// Rs232Arguments class
    /// </summary>
    public class Rs232Arguments
    {
        private string RECEIVED_DATA_STRING;
        private byte[] RECEIVED_DATA_ARRAY;
        /// <summary>
        /// property represents received data string
        /// </summary>
        public string ReceivedString
        {
            get { return RECEIVED_DATA_STRING; }
        }
        /// <summary>
        /// property represents received data byte array
        /// </summary>
        public byte[] ReceivedArray
        {
            get { return RECEIVED_DATA_ARRAY; }
        }
        /// <summary>
        /// constructor 
        /// </summary>
        /// <param name="RECEIVED_DATA_STRING">received data string</param>
        public Rs232Arguments(string RECEIVED_DATA_STRING)
        {
            this.RECEIVED_DATA_STRING = RECEIVED_DATA_STRING;
            this.RECEIVED_DATA_ARRAY = ASCIIEncoding.ASCII.GetBytes(RECEIVED_DATA_STRING);
        }
    }

    /// <summary>
    /// delegate used in event hadling
    /// </summary>
    /// <param name="Sender">sender object</param>
    /// <param name="e">arguments</param>
    public delegate void MonitorDelegate(object Sender, Rs232Arguments e);

    /// <summary>
    /// Rs232 Class
    /// </summary>
    public class Rs232
    {
        private string _COM_PORT;
        private int _BAUD_RATE;
        private Parity _PARITY;
        private StopBits _STOP_BITS;
        private int _DATA_BITS;
        private SerialPort _SERIAL;
        private Thread _MONITOR_THREAD;
        private bool _MONITOR_RUNNING;

        /// <summary>
        /// monitor event is fired when data is recived, note: if monitor is enabled dont call the read function
        /// </summary>
        public event MonitorDelegate OnMonitor;
        protected virtual void OnMonitorEvent(Rs232Arguments e)
        {
            OnMonitor(this, e);
        }

        /// <summary>
        /// Constructor initialzes serial communication at 9600,none parity,one stop bit, 8bits data
        /// </summary>
        /// <param name="PortName">PortName ex: COM1</param>
        public Rs232(string PortName)
        {
            _COM_PORT = PortName;
            _BAUD_RATE = 9600;
            _PARITY = Parity.None;
            _DATA_BITS = 8;
            _STOP_BITS = StopBits.One;
            _SERIAL = new SerialPort(_COM_PORT, _BAUD_RATE, _PARITY, _DATA_BITS, _STOP_BITS);
            _MONITOR_RUNNING = false;
        }
        /// <summary>
        /// Constructor initialzes serial communication at specified parameters
        /// </summary>
        /// <param name="PortName">PortName ex: COM1</param>
        /// <param name="BaudRate">BaudRate ex: 9600 bits/second</param>
        /// <param name="ParityOption">Parity error check</param>
        /// <param name="DataBits">number of data bits to be transmitted and received ex: 8 bits</param>
        /// <param name="StopBitsOption">Stop Bits ex: One stopbit</param>
        public Rs232(string PortName, int BaudRate, Parity ParityOption, int DataBits, StopBits StopBitsOption)
        {
            _COM_PORT = PortName;
            _BAUD_RATE = BaudRate;
            _PARITY = ParityOption;
            _DATA_BITS = DataBits;
            _STOP_BITS = StopBitsOption;
            _SERIAL = new SerialPort(_COM_PORT, _BAUD_RATE, _PARITY, _DATA_BITS, _STOP_BITS);
            _MONITOR_RUNNING = false;
        }
        /// <summary>
        /// sends a string through serial port
        /// </summary>
        /// <param name="Data">string to send</param>
        public void Send(string Data)
        {
            byte[] _data = ASCIIEncoding.ASCII.GetBytes(Data);
            if (!_MONITOR_RUNNING)
            {
                _SERIAL.Open();
                _SERIAL.Write(_data, 0, _data.Length);
                _SERIAL.Close();
            }
            else
            {
                _SERIAL.Write(_data, 0, _data.Length);
            }
        }
        /// <summary>
        /// returns a string read from the serial port
        /// </summary>
        /// <param name="ReadSize">number of bytes to read</param>
        /// <returns>data string or TimeOut if occured</returns>
        public string Read(int ReadSize)
        {
            byte[] _data = new byte[ReadSize];
            if (!_MONITOR_RUNNING)
            {
                _SERIAL.ReadTimeout = 1000;
                try
                {
                    _SERIAL.Open();
                    _SERIAL.Read(_data, 0, _data.Length);
                    _SERIAL.Close();
                }
                catch (Exception)
                {
                    _SERIAL.Close();
                    return "TimeOut";
                }
                return (ASCIIEncoding.ASCII.GetString(_data));
            }
            return "MonitorRunning";
        }
        /// <summary>
        /// returns a string read from the serial port
        /// </summary>
        /// <param name="ReadSize">number of bytes to read</param>
        /// <param name="TimeOut">Read Time Out</param>
        /// <returns>data string or TimeOut if occured</returns>
        public string Read(int ReadSize, int TimeOut)
        {
            if (!_MONITOR_RUNNING)
            {
                byte[] _data = new byte[ReadSize];
                _SERIAL.ReadTimeout = TimeOut;
                try
                {
                    _SERIAL.Open();
                    _SERIAL.Read(_data, 0, _data.Length);
                    _SERIAL.Close();
                }
                catch (Exception)
                {
                    _SERIAL.Close();
                    return "TimeOut";
                }
                return (ASCIIEncoding.ASCII.GetString(_data));
            }
            return "MonitorRunning";
        }
        /// <summary>
        /// returns if serial port is open
        /// </summary>
        /// <returns>true if open else false</returns>
        public bool IsPortOpen()
        {
            return (_SERIAL.IsOpen);
        }
        /// <summary>
        /// returns if serial port is closed
        /// </summary>
        /// <returns>true if closed else false</returns>
        public bool IsPortClosed()
        {
            return (!_SERIAL.IsOpen);
        }
        /// <summary>
        /// returns if monitor is running
        /// </summary>
        /// <returns>true if running else false</returns>
        public bool IsMonitorRunning()
        {
            return _MONITOR_RUNNING;
        }
        /// <summary>
        /// starts the monitor
        /// </summary>
        /// <param name="ReadSize">data size</param>
        public void StartMonitor(int ReadSize)
        {
            _MONITOR_RUNNING = true;
            _MONITOR_THREAD = new Thread(new ParameterizedThreadStart(Monitor));
            _MONITOR_THREAD.Start(ReadSize);
        }
        /// <summary>
        /// stops the monitor
        /// </summary>
        public void StopMonitor()
        {
            _MONITOR_RUNNING = false;
        }
        private void Monitor(object ReadSize)
        {
            string Data;
            byte[] _data = new byte[(int)ReadSize];
            _SERIAL.ReadTimeout = 100;
            _SERIAL.Open();
            while (_MONITOR_RUNNING)
            {
                try
                {
                    _SERIAL.Read(_data, 0, _data.Length);
                    Data = ASCIIEncoding.ASCII.GetString(_data);
                    OnMonitorEvent(new Rs232Arguments(Data));
                }
                catch (Exception)
                {
                    ;
                }
            }
            _SERIAL.Close();
        }

    }
}
